package com.javaxing.mallorder.service.impl;

import com.javaxing.mallorder.feign.StockFeignService;
import com.javaxing.mallorder.feign.UserFeignService;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import com.javaxing.mallorder.entity.SysOrder;
import com.javaxing.mallorder.mapper.SysOrderMapper;
import com.javaxing.mallorder.service.SysOrderService;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;

@Service
public class SysOrderServiceImpl implements SysOrderService{

    @Resource
    private SysOrderMapper sysOrderMapper;
    @Resource
    private StockFeignService stockFeignService;

    @Resource
    private UserFeignService userFeignService;

    @Override
    public int deleteByPrimaryKey(Integer orderId) {
        return sysOrderMapper.deleteByPrimaryKey(orderId);
    }

    @Override
    public int insert(SysOrder record) {
        return sysOrderMapper.insert(record);
    }

    @Override
    public int insertSelective(SysOrder record) {
        return sysOrderMapper.insertSelective(record);
    }

    @Override
    public SysOrder selectByPrimaryKey(Integer orderId) {
        return sysOrderMapper.selectByPrimaryKey(orderId);
    }

    @Override
    public int updateByPrimaryKeySelective(SysOrder record) {
        return sysOrderMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByPrimaryKey(SysOrder record) {
        return sysOrderMapper.updateByPrimaryKey(record);
    }

    @Override
    @Transactional
    // 开启分布式全局事务，名称为 createOrder
    @GlobalTransactional(name="createOrder",rollbackFor=Exception.class)
    public void createOrder(Integer shopId,Integer count, BigDecimal price, String title, Integer userId) {
        // 创建订单
        SysOrder order = new SysOrder();
        order.setShopId(shopId);
        order.setPrice(price);
        order.setStatus(1);
        order.setTitle(title);
        order.setUserId(userId);
        sysOrderMapper.insertSelective(order);

        // 扣减库存
        stockFeignService.reduceStock(shopId,count);
//
//        // 扣除余额
        userFeignService.reduceBalance(userId,price);
    }
}
